@@ -3,7 +3,7 @@
 
 /*
 * this file is part of the oxygen gtk engine
-* Copyright (c) 2010 Hugo Pereira Da Costa <hugo@oxygen-icons.org>
+* Copyright (c) 2010 Hugo Pereira Da Costa <hugo.pereira@free.fr>
 *
 * based on the Null Theme Engine for Gtk+.
 * Copyright (c) 2008 Robert Staudinger <robert.staudinger@gmail.com>
@@ -61,12 +61,6 @@ namespace Oxygen
         //! text editor
         GtkWidget* _textView;
 
-        //! flat mode changed
-        Signal _flatModeChangedId;
-
-        //! toggle wrap mode
-        Signal _wrapModeChangedId;
-
     };
 
 }
