@@ -1,6 +1,7 @@
 /*
 * this file is part of the oxygen gtk engine
-* Copyright (c) 2010 Hugo Pereira Da Costa <hugo@oxygen-icons.org>
+* Copyright (c) 2010 Hugo Pereira Da Costa <hugo.pereira@free.fr>
+* Copyright (c) 2010 Ruslan Kabatsayev <b7.10110111@gmail.com>
 *
 * This  library is free  software; you can  redistribute it and/or
 * modify it  under  the terms  of the  GNU Lesser  General  Public
@@ -99,44 +100,47 @@ namespace Oxygen
         const bool animationsEnabled( settings.animationsEnabled() );
 
         // pass animations configuration to engines
-        widgetStateEngine().setApplicationName( settings.applicationName() );
-        widgetStateEngine().setEnabled( animationsEnabled && settings.genericAnimationsEnabled() );
-        widgetStateEngine().setDuration( settings.genericAnimationsDuration() );
+        _widgetStateEngine->setApplicationName( settings.applicationName() );
+        _widgetStateEngine->setEnabled( animationsEnabled && settings.genericAnimationsEnabled() );
+        _widgetStateEngine->setDuration( settings.genericAnimationsDuration() );
 
-        arrowStateEngine().setApplicationName( settings.applicationName() );
-        arrowStateEngine().setEnabled( animationsEnabled && settings.genericAnimationsEnabled() );
-        arrowStateEngine().setDuration( settings.genericAnimationsDuration() );
+        _arrowStateEngine->setApplicationName( settings.applicationName() );
+        _arrowStateEngine->setEnabled( animationsEnabled && settings.genericAnimationsEnabled() );
+        _arrowStateEngine->setDuration( settings.genericAnimationsDuration() );
 
-        scrollBarStateEngine().setApplicationName( settings.applicationName() );
-        scrollBarStateEngine().setEnabled( animationsEnabled && settings.genericAnimationsEnabled() );
-        scrollBarStateEngine().setDuration( settings.genericAnimationsDuration() );
+        _scrollBarStateEngine->setApplicationName( settings.applicationName() );
+        _scrollBarStateEngine->setEnabled( animationsEnabled && settings.genericAnimationsEnabled() );
+        _scrollBarStateEngine->setDuration( settings.genericAnimationsDuration() );
 
-        tabWidgetStateEngine().setApplicationName( settings.applicationName() );
-        tabWidgetStateEngine().setEnabled( animationsEnabled && settings.genericAnimationsEnabled() );
-        tabWidgetStateEngine().setDuration( settings.genericAnimationsDuration() );
+        _tabWidgetStateEngine->setApplicationName( settings.applicationName() );
+        _tabWidgetStateEngine->setEnabled( animationsEnabled && settings.genericAnimationsEnabled() );
+        _tabWidgetStateEngine->setDuration( settings.genericAnimationsDuration() );
 
-        treeViewStateEngine().setApplicationName( settings.applicationName() );
-        treeViewStateEngine().setEnabled( animationsEnabled && settings.genericAnimationsEnabled() );
-        treeViewStateEngine().setDuration( settings.genericAnimationsDuration() );
+        _treeViewStateEngine->setApplicationName( settings.applicationName() );
+        _treeViewStateEngine->setEnabled( animationsEnabled && settings.genericAnimationsEnabled() );
+        _treeViewStateEngine->setDuration( settings.genericAnimationsDuration() );
 
-        menuBarStateEngine().setApplicationName( settings.applicationName() );
-        menuBarStateEngine().setAnimationsEnabled( animationsEnabled && (settings.menuBarAnimationType() != None) );
-        menuBarStateEngine().setFollowMouse( settings.menuBarAnimationType() == FollowMouse );
-        menuBarStateEngine().setDuration( settings.menuBarAnimationsDuration() );
-        menuBarStateEngine().setFollowMouseAnimationsDuration( settings.menuBarFollowMouseAnimationsDuration() );
+        _menuBarStateEngine->setApplicationName( settings.applicationName() );
+        _menuBarStateEngine->setAnimationsEnabled( animationsEnabled && (settings.menuBarAnimationType() != None) );
+        _menuBarStateEngine->setFollowMouse( settings.menuBarAnimationType() == FollowMouse );
+        _menuBarStateEngine->setDuration( settings.menuBarAnimationsDuration() );
+        _menuBarStateEngine->setFollowMouseAnimationsDuration( settings.menuBarFollowMouseAnimationsDuration() );
 
-        menuStateEngine().setApplicationName( settings.applicationName() );
-        menuStateEngine().setEnabled( animationsEnabled && (settings.menuAnimationType() != None) );
-        menuStateEngine().setFollowMouse( settings.menuAnimationType() == FollowMouse );
-        menuStateEngine().setDuration( settings.menuAnimationsDuration() );
-        menuStateEngine().setFollowMouseAnimationsDuration( settings.menuFollowMouseAnimationsDuration() );
+        _menuStateEngine->setApplicationName( settings.applicationName() );
+        _menuStateEngine->setEnabled( animationsEnabled && (settings.menuAnimationType() != None) );
+        _menuStateEngine->setFollowMouse( settings.menuAnimationType() == FollowMouse );
+        _menuStateEngine->setDuration( settings.menuAnimationsDuration() );
+        _menuStateEngine->setFollowMouseAnimationsDuration( settings.menuFollowMouseAnimationsDuration() );
 
         // for now, only Fade animations mode is supported for toolbar animations
-        toolBarStateEngine().setApplicationName( settings.applicationName() );
-        toolBarStateEngine().setEnabled( animationsEnabled && (settings.toolBarAnimationType() != None) );
-        toolBarStateEngine().setFollowMouse(  settings.toolBarAnimationType() == FollowMouse );
-        toolBarStateEngine().setDuration( settings.genericAnimationsDuration() );
-        toolBarStateEngine().setFollowMouseAnimationsDuration( settings.toolBarAnimationsDuration() );
+        _toolBarStateEngine->setApplicationName( settings.applicationName() );
+        _toolBarStateEngine->setEnabled( animationsEnabled && (settings.toolBarAnimationType() != None) );
+        _toolBarStateEngine->setFollowMouse(  settings.toolBarAnimationType() == FollowMouse );
+        _toolBarStateEngine->setDuration( settings.genericAnimationsDuration() );
+        _toolBarStateEngine->setFollowMouseAnimationsDuration( settings.toolBarAnimationsDuration() );
+
+        // background hint engine
+        _backgroundHintEngine->setUseBackgroundGradient( settings.useBackgroundGradient() );
 
     }
 
@@ -148,7 +152,7 @@ namespace Oxygen
         // https://bugzilla.gnome.org/show_bug.cgi?id=643416
         #if ENABLE_INNER_SHADOWS_HACK
         if(!getenv("OXYGEN_DISABLE_INNER_SHADOWS_HACK"))
-            _innerShadowHook.connect( "realize", (GSignalEmissionHook)innerShadowHook, this );
+        { _innerShadowHook.connect( "realize", (GSignalEmissionHook)innerShadowHook, this ); }
         #endif
 
         _sizeAllocationHook.connect( "size-allocate", (GSignalEmissionHook)sizeAllocationHook, this );
@@ -224,11 +228,14 @@ namespace Oxygen
         // check type
         if( !GTK_IS_WIDGET( widget ) ) return FALSE;
 
+        // cast data
+        Animations& animations( *static_cast<Animations*>(data) );
+
         // groupbox labels
         #if ENABLE_GROUPBOX_HACK
-        if( static_cast<Animations*>( data )->groupBoxLabelEngine().contains( widget ) )
+        if( animations.groupBoxLabelEngine().contains( widget ) )
         {
-            static_cast<Animations*>( data )->groupBoxLabelEngine().adjustSize( widget );
+            animations.groupBoxLabelEngine().adjustSize( widget );
             return TRUE;
         }
         #endif
@@ -240,7 +247,6 @@ namespace Oxygen
         GtkWindow* window( GTK_WINDOW( widget ) );
         if( gtk_window_get_type_hint( window ) != GDK_WINDOW_TYPE_HINT_COMBO ) return TRUE;
 
-        Animations& animations( *static_cast<Animations*>(data) );
         GtkWidget* combobox = animations.comboBoxEngine().find( widget );
         if( !combobox ) combobox = animations.comboBoxEntryEngine().find( widget );
         if( !combobox ) combobox = animations.comboEngine().find( widget );
@@ -281,7 +287,9 @@ namespace Oxygen
         Animations& animations( *static_cast<Animations*>(data) );
         if( !animations.innerShadowsEnabled() ) return TRUE;
 
+        // blacklist
         if( Gtk::g_object_is_a( G_OBJECT( widget ), "SwtFixed" ) ) return TRUE;
+        if( Gtk::g_object_is_a( G_OBJECT( widget ), "GtkPizza" ) ) return TRUE;
 
         GtkWidget* parent(gtk_widget_get_parent(widget));
         if( !GTK_IS_SCROLLED_WINDOW( parent ) ) return TRUE;
@@ -341,8 +349,9 @@ namespace Oxygen
                 gtk_frame_set_shadow_type( frame, GTK_SHADOW_OUT );
 
                 // register to engine
-                static_cast<Animations*>( data )->groupBoxLabelEngine().registerWidget( widget );
-                static_cast<Animations*>( data )->groupBoxLabelEngine().adjustSize( widget );
+                Animations& animations( *static_cast<Animations*>(data) );
+                animations.groupBoxLabelEngine().registerWidget( widget );
+                animations.groupBoxLabelEngine().adjustSize( widget );
 
             }
 
